/**\file player.c
 * VLSI Modular Player Main file
 *
 *
 * This file contains the main() function.
 * It's also the main playground for player UI development.
 * Many functions will be transferred to a new module when they're ok.
 */

/** \mainpage
 * 
 * This is a preliminary version of the new MP3 player 
 * evaluation kit source code. Some bugs still exist.
 * 
 * This is pure C code. No inline assembly.
 * It compiles with SDCC 2.4.0. Upload with Atmel Flip.
 * Connect a terminal with settings 115200bps,N81 to the
 * serial port to see the console output.
 *
 * The code is written for VLSI Evakit 1.5-1.6 that has VS1011/1002
 * and AT89C51RD2 microcontroller. 
 * Now also Evakit 2.0-2.5 with VS1011, VS1002 or VS1003 is supported. 
 * It should be easily portable
 * for other microcontrollers. 
 *
 * All microcontroller/pcb -specific definitions should
 * be in board.h and board.c but some board specific
 * features may still lurk in the code elsewhere. 
 * Due to SDCC requirements, interrupts are defined in player.c (main file).
 * They are of course board/cpu dependent.
 *
 * Throughout the code, SDCC memory allocation pragmas "code" "data" and "xdata"
 * are used, sorry. "Code" allocates from ROM, "data" allocates from fast RAM,
 * "xdata" allocates from slow RAM. You may need to remove/adjust these to make
 * code compatible with your compiler.
 *
 * It is strongly suggested that you use the free SDCC 2.4.0 to compile the
 * code. SDCC 2.5.0 fails to compile the code: Although it has advantages
 * over v2.4, it tries to allocate more extra temporary memory than is
 * available if you compile all the modules (incl. recording).
 *
 * <B>How to get started</b>
 *
 * - Look at "File list" to see what kind of modules are in the project.
 * - Look at vs10xx.c and vs10xx.h file documentations to see how the
 * vs10xx is reset and how to send MP3 data to it (PlayDiskSectors()).
 * - Function AvailableProcessorTime() handles the user interface. It is
 * called by PlayDiskSectors() whenever the player seems to have some free
 * time. Additionally, a timer interrupt routine sets the 
 * <b>userInterfaceDelayElapsed</b> flag every 50ms or so. To avoid unnecessary
 * CPU load, AvailableProcessorTime()
 * checks this flag and updates display etc only when this flag is set.
 * - The user interface commands the player routine by setting a special
 * state variable <b>playingState</b>. For instance if the user pushes a "Next Song"
 * button, the ui should set <b>playingState</b> to <i>PS_NEXT_SONG</i>. 
 * PlayDiskSectors() detects this, stops playing and returns to caller.
 * - Much of the main functionality is still in player.c and main(). Much
 * of it will be transferred to a new file ui.c at some time.
 *
 * The description of how the program works and the module structure will be
 * written here soon. Please check http://www.vlsi.fi/
 * often to see if the final code is released. Email to
 * mp3@vlsi.fi about bugs.
 *
 * <b>Changes in version 0.5:</b>
 *
 * Filesystem and Storage are affected. Code has been added to handle new
 * MMC cards with manufacturer id 21 and possibly others that seem to
 * (sometimes) require resetting CRC calculation before they agree to work.
 * Storage.c is divided into submodules storage.c and mmc.c and a new 
 * file buffer.c is created to hold the global I/O buffer. This clarifies
 * the code a lot. Lots of documentation is added to MMC and filing system
 * functions, do yourself a favor and read it through.
 *
 * In storage.c, mmc.c and filesys.c a "dummy" declarator Public is added
 * in declarations for functions that are called from outside the module
 * and thus form module interface. It has no code effect in C but is a
 * reminder for the programmer that the functions need to be adjusted
 * carefully.
 *
 * Storage.c is now a "middle-level" disk handler that takes advantage of
 * lower level functions in mmc.c. It handles decisions of when to seek
 * based on testing whether the underlying storage module supports 
 * separate seeks before reads. A new function PrepareToReadDiskSector() is
 * for telling the storage that it can do a seek now if the storage
 * supports it.
 *
 * Adding support for different storage media that MMC is easy: only mmc.c
 * needs to be rewritten. Currently it only has 3 public functions:
 * Init, Seek sector and Read sector. As the code uses ONLY the address
 * granularity of 512 bytes, called a "disk sector", supporting hard
 * disks etc should be very simple.
 *
 *
 * <b>Support for compiling under Windows:</b>
 *
 * The code is natively developed under Linux and Makefile for GNU Make is
 * included. Now also .bat files for compiling in Dos window under Win32
 * are included, plus a readme.txt file. Use them at your own risk :)
 *
 * <B>Changes in version 0.6</b>
 *
 * The major improvement is adding recording support. Recording for 
 * VS1003 is supported. See the Record() function documentation for 
 * instructions. The Record() function can only create a single file
 * fragment. If a fragment boundary is reached, recording continues
 * to a new file.
 * Also there is an experimental function PlayAVI() in
 * module aviplay.c that often succeeds in demuxing an avifile and
 * sending just the audio stream for decoding.
 *
 * Much of the FAT code is rewritten for speedups.
 *
 * \todo VFAT long file names,
 * 
 * \remarks If you are usind SDCC 2.4.0 and you need just a little bit
 * of more DSEG memory (now how on Earth that might happen...) you
 * can hand-allocate global variables to unused addresses 0x08..0x1F,
 * for instance: declaring
 * <pre>
 * data at 0x08 unsigned int myVariable;
 * </pre>
 * would place myVariable in DSEG addresses 0x08..0x09.
 *
 *
 * <b> Changes in version 0.61 </b>
 * MMC recording routine: realtime bug fixed, writing to mmc card is
 * a little bit more reliable now
 *
 * Note for HyperTerminal users: Check the "Append Line Feeds" checkbox 
 * in terminal settings -> ascii setup.
 *
 * Example output to console: When the code runs it prints out something like:
 * (from a little older version)
 * <pre>
 *
 * Board: Console is up.
 * Init: Board; supports: AT89C51RD2, MMC, VS10xx, RS232
 * 
 * VLSI Modular Player
 * Starting up.
 * 
 * Init: Filesystem; supports: MBR, FAT16, FAT32
 * Trying to bring up storage...
 * Init: Storage v2: supports: MMC,SD,miniSD in SPI mode
 * Init: MMC
 * Card found, starting... Ok.
 * 
 * MMC Manufacturer#: 30 Product Name: MMC    Production date: 10 / 2004
 * Media serial number: 00000468
 * This MMC has no support for interrupted read. Using compatibility mode.
 * Storage initialized in seek-and-read mode.
 * Storage ok.
 * Sector 0 read...ok.
 * Sector signature...ok.
 * Partition 1... at sector 00000020
 * Formatter signature: %j!BFIHC
 * Bytes per sector:    512
 * Bytes per sector:    512
 * Reserved sectors:    1
 * Number of FATs:      2
 * Root entries:        512
 * Total Sectors 16:    0
 * Fat Size 16:         123
 * Hidden Sectors:      32
 * Total Sectors 32:    125408
 * File system is FAT16
 * File system signature is FAT16
 * Volume Label is NO NAME
 * Sectors per Cluster: 4
 * FAT Start sector:    33
 * Root start sector:   279
 * Data start sector:   303
 * 
 * Init: VS10XX
 * Loading patch.
 * 
 * Filesystem: Looking for file 1 ... found, FAT name is "UTOPIA~1MP3"
 * File size: 1471792 bytes. Start cluster: 0003h, sector 315 decimal.
 * Building file fragment table...
 * Fragment Start: 315  Fragment End: 3190  Fragment length: 2876
 * 
 * Fragments: 1
 * Player: Playing from sector 315
 *
 * </pre>
 * ...and so on.
 *
 

*/




#include "board.h"
#include "lcd.h"
#include "filesys.h"
#include "vs10xx.h"
#include "storage.h"
#include "display.h"
#include "string.h"
#include "ui.h"
#include "mmc.h"
#include "record.h"


/**  typedef for loading patch code */
#define u_int8 code unsigned char 
/**  typedef for loading patch code */
#define u_int16 code unsigned int 

//include a patch code module
#ifdef VS1003
#include "spec_rew.c"
#else
#include "loud_sa.c"
#endif



/** How much to wait for button pressed before returning to title */
#define UI_RETURN_DELAY 150

/** Playing State Global */
xdata playingstatetype playingState = PS_NORMAL;

/** Current interrupt level */
unsigned char intlevel = 0;

/** Interrupt-driven flag to reduce unnecessary UI updates.
 * This is set to 1 by timer interrupt every n milliseconds. */
xdata unsigned char userInterfaceDelayElapsed = 0;

/** Current User Interface mode (Title, Volume, ...) */
xdata uimodetype uiMode = UI_SPEC;

xdata unsigned char SPMax=0;



/** User Interface Timer interrupt.
 * Does NOT do any screen/button handling, only updates
 * screen scroll variable and the 
 * userInterfaceDelayElapsed flag. */
void timer0_interrupt(void) interrupt 1{
  static char displayDelay;
  userInterfaceDelayElapsed = 1;
  if (++displayDelay==12){
    displayDelay = 0;
    displayPosition++;
    if (displayPosition>displayLength+7){
      displayPosition = -7;
    }
  }
  if (SP>SPMax) SPMax = SP;
  

}




/** Load a patch code to VS10xx */
void LoadPatch(){
  ConsoleWrite ("Loading patch.\r");
  for (temp.i=0; temp.i < CODELEN; temp.i++) {    
    Mp3WriteRegister(atab[temp.i], dtab[temp.i]>>8, dtab[temp.i]&0xff);
   }
  Delay(10);
}

/** Start patch code */
void StartPatch()
{

#ifdef VS1003
  //  Mp3WriteRegister(SPI_AIADDR, 0x00, 0x50);            
#else
  //Mp3WriteRegister(SPI_AIADDR, 0x02, 0x00);            
  //Mp3WriteRegister(SPI_AICTRL0, 0x00, 0x00);
  //Mp3WriteRegister(SPI_AICTRL0, 0x00, 0x01);
#endif
}




/** Sets playing time and mode icon to left display title */
void ScreenSetPlayTime(){
  unsigned int playTime;
  unsigned char minutes, seconds;

  playTime = Mp3ReadRegister(SPI_DECODE_TIME);
  minutes = playTime/60;
  seconds = playTime%60;
  displayTitle[0]=('0'+minutes/10);
  displayTitle[1]=('0'+minutes%10);
  displayTitle[2]=(':');
  displayTitle[3]=('0'+seconds/10);
  displayTitle[4]=('0'+seconds%10);
  displayTitle[5]=(' ');
  displayTitle[6]=(' ');
  displayTitle[7]=(' ');
  
  if (seconds&1){
    if (playingState == PS_RECORDING){
      displayTitle[6]='o';
    }else{
      displayTitle[6]='\x7e';
    }

  }

  if ((playingState != PS_NORMAL)&&(playingState != PS_RECORDING)){
    displayTitle[6]='\xdb';
  }
  
  

}







/** This function is called when the player is playing a song
 and there is free processor time. The basic task of this
 function is to implement the player user interface. */
void AvailableProcessorTime(){

  /** Local variables: */

  /** - <b>volume</b>: Current volume setting */
  static xdata unsigned char volume = 20;

  /** - <b>bass</b>: Current bass enhancer setting */
  static xdata unsigned char bass = 0;

  /** - <b>treble</b>: Current treble enhancer setting */
  static xdata unsigned char treble = 64; //middle setting = 0

  /** - <b>reclevel</b>: Current recording level setting */
  static xdata unsigned char reclevel = 50;

  unsigned int i; //temp variable

  static bit bassUpdateNeeded;

  /** - <b>uiReturnDelay</b>: Counts down to returning to UI mode
      0 (title screen) */

  static unsigned char uiReturnDelay=0;


#if 0
  // Simple serial control demo: receiving 'N' advances to next song
  // and 'P' to previous song

  if (RI) { //Serial Data Received
    RI = 0; //Clear received flag!!!

    i = SBUF; // read received character from uart buffer to temp i

    if (i=='N'){
      playingState = PS_NEXT_SONG;
    }
    if (i=='P'){
      playingState = PS_PREVIOUS_SONG;
    }
  }
#endif




  if (!userInterfaceDelayElapsed){
    return; /* Not yet time to do user interface stuff */
  }
  
  userInterfaceDelayElapsed = 0;


  // User Interface Modes
  // Return to UI_TITLE after long period of button inactivity

  if (uiReturnDelay == 0){
    if ((uiMode != UI_SPEC)&&(uiMode != UI_CUE))
      uiMode = UI_TITLE;
    else{
    }
  }else{
    uiReturnDelay--;
  }

  

#ifndef VS1003
  if (uiMode==UI_SPEC) //Only VS1003 has spectrum analyzer
    uiMode++;
  if (uiMode==UI_RECLEVEL) //Only record in VS1003 chips
    uiMode++;
#endif




  if (uiMode==UI_INFO)
    uiMode++; //Nothing interesting currently in screen UI_INFO.
  if (uiMode==UI_STOP)
    uiMode++; //Nothing interesting currently in screen UI_STOP.

  /* Screen updates */
  switch(uiMode){
    
  case UI_TITLE:  //Playing time, song title

    displayStyle = DS_TEXT;
    ScreenSetPlayTime();
    
    if (playingState==PS_RECORDING){      
      displayStyle = DS_PERCENTAGE;
    }     
    
    break;
    
  case UI_STOP: //Ask if the user wants to stop
    displayStyle = DS_NO_YES;
    strcpy (displayTitle, "Stop?   ");
    break;

  case UI_CUE: //Ask if the user wants to fast forward or rewind
    displayStyle = DS_RAWTEXT;
    strcpy (displayTitle, "REW/CUE  <-   ->");

    //i = Mp3ReadRegister(SPI_HDAT1);
    //displayTitle[5] = lcd_hexchars[i>>12];
    //displayTitle[6] = lcd_hexchars[(i>>8)&0x0f];
    //displayTitle[7] = lcd_hexchars[(i>>4)&0x0f];
    //displayTitle[8] = lcd_hexchars[i&0x0f];

    break;
    
  case UI_VOLUME: //Show volume bar
    LcdSelectFont(lcdFont_barchars);
    displayStyle = DS_PERCENTAGE;
    strcpy(displayTitle,"Volume: ");
    displayValue = (100-(volume>>1));
    if (displayValue<0) displayValue = 0;
    break;

  case UI_BASS: //Show bass bar
    LcdSelectFont(lcdFont_barchars);
    displayStyle = DS_PERCENTAGE;
    strcpy(displayTitle,"Bass:   ");
    displayValue = (bass/5)*4;
    if (displayValue<1) displayValue = 1;
    break;

  case UI_TREBLE: //Show treble bar
    LcdSelectFont(lcdFont_barchars);
    displayStyle = DS_PERCENTAGE;
    strcpy(displayTitle,"Treble: ");
    displayValue = (treble/5)*4;
    if (displayValue<1) displayValue = 1;
    break;

  case UI_RECLEVEL: //Show recording level

    if (playingState==PS_RECORDING){
      LcdSelectFont(lcdFont_barchars);
      displayStyle = DS_PERCENTAGE;
      strcpy(displayTitle,"RecLevel");
      displayValue = (reclevel);
      if (reclevel==0){
	strcpy(displayTitle,"RecLevel <Auto> ");
	displayStyle = DS_RAWTEXT; //Reveal the "auto" text
      }
    }else{
      //Not in recording
      displayStyle = DS_NO_YES;
      strcpy (displayTitle, "Record? ");
    }
    break;


  case UI_SPEC: // Spectrum analyzer

    if (playingState == PS_RECORDING){
      uiMode++; //Don't show specana when recording
    }

    LcdSelectFont(lcdFont_vertical_barchars);
    displayStyle = DS_RAWTEXT;
    ScreenSetPlayTime();

    // Request to read Spectrum Analyzer register
    Mp3WriteRegister(7, 0x18, 0x04);

    // Read Spectrum Analyzer registers
    {
      for (i=0;i<8;i++) { //for each band i in [0..7]
	signed char thisband = Mp3ReadRegister(6);
	
	/* thisband now has spectrum value 0..63. We adjust it a little
	   for best image on the evakit LCD. You also get peak values
	   etc from the chip but we don't use them in the evakit screen.
	   we use 8 bands, but up to 23 are obtainable from the chip. */

	thisband &= 63;                /* mask all but level info */
	thisband >>= 1;                /* LCD resolution is not high */
	thisband -= 3;                 /* base offset */ 
	if (i==0) thisband -= 2;       /* decrease bass bar level */
	if (i==6) thisband += 1;       /* increase treble bar level */
	if (i==7) thisband += 1;       /* increase treble bar level */
	if (thisband>7) thisband = 7;  /* high limit */
	if (thisband<0) thisband =' '; /* low limit */

	displayTitle[i+8] = thisband; //Store to display mem
	
      }//for each band
    }
    break;
    
  } //switch(uiMode)    

  // Buttons handler
  // Perform mode-dependent button handling

  if ((KEY_BUTTON) && (uiReturnDelay < UI_RETURN_DELAY-20)){
    LcdReset();
    uiMode++;
    uiReturnDelay = UI_RETURN_DELAY;
  }

  switch(uiMode){

  case UI_TITLE:
  case UI_SPEC:

    if (KEY_FARRIGHT){
      playingState = PS_NEXT_SONG; /* Request */
    }      
    if (KEY_FARLEFT){
      playingState = PS_PREVIOUS_SONG; /* Request */
    }      
    break;


  case UI_VOLUME:
    
    if (KEY_FARLEFT){
      uiReturnDelay = UI_RETURN_DELAY;
      if (volume++ == 254) volume = 254; //Change + limit to 254 (minimum vol)
      Mp3SetVolume(volume,volume);
    }
    
    if (KEY_FARRIGHT){
      uiReturnDelay = UI_RETURN_DELAY;
      if (volume-- == 0) volume = 0; //Change + limit to 0 (maximum volume)
      Mp3SetVolume(volume,volume);
    }
    break; // UI_VOLUME

  case UI_BASS:
  case UI_TREBLE: //BASS and TREBLE use the same VS10xx register
    bassUpdateNeeded = 0;
    
    //First let's see if a key is pressed; does the user want to set level?

    if (uiMode==UI_BASS){ //BASS screen is active

      if (KEY_FARLEFT){
	bassUpdateNeeded = 1;
	if (bass-- == 0) bass = 0; //Change + limit to 0 (OFF setting)
      }
      if (KEY_FARRIGHT){
	bassUpdateNeeded = 1;
	if (bass++ == 127) bass = 127; //Change + limit to 127 (max setting)
      }

    }else{ //TREBLE screen is active
      
      if (KEY_FARLEFT){
	bassUpdateNeeded = 1; //SCI_BASS is for both bass and treble
	if (treble-- == 0) treble = 0; //Change + limit to 0 (OFF setting)
      }
      if (KEY_FARRIGHT){
	bassUpdateNeeded = 1; //SCI_BASS is for both bass and treble
	if (treble++ == 127) treble = 127; //Change + limit to 127 (max)
      }
      
    }


    if (bassUpdateNeeded){
      unsigned int newBassRegister;

      //User has pushed button to alter bass/treble register
      //calculate new value

      /** Bass/Treble setting calculation 
       *
       * There are 2 or 4 adjustments possible:
       * - Bass Frequency (1011B/E, 1002, 10x3) : 2..15 -> 20..1500 Hz
       * - Bass boost (1011B/E, 1002, 10x3) : 0..15 -> 0ff..+15dB)
       * - Treble Frequency (1011E, 10x3) : 0..15 -> 0..15kHz
       * - Treble level (1011E, 10x3) : -8..+7 -> -12..+10.5dB
       * 
       * In this software, all are adjusted based from a single setting
       * (0..128) for bass and another for treble. You may wish to allow
       * user to control frequency and level separately or set fixed
       * frequencies for a known speaker type. This algorithm increases
       * the frequency range of adjustment with the increasing level.
       */
       
      //Let's start from bass frequency setting.
      //min(0) should give 2 (20Hz), max(127) should give 15 (150Hz)
      newBassRegister = (bass + 23) / 10; //into bits 3..0, clear hibits
      
      //Bass boost level.
      //min(0) should give 0, max(127) should give 15
      newBassRegister |= (bass>>3)<<4; //insert to bits 7..4

      //Then the treble frequency
      //min(0) should give 15(15kHx), max(127) should give 2(2kHz)
      newBassRegister |= (((148-treble)>>3)+2)<<8; //insert into bits 11..8

      //Finally the treble value (-8..7)
      //min(0) should give -8, max(127) should give 7;
      newBassRegister |= ((treble>>3)-8)<<12; //insert into bits 15..12
      
      
      uiReturnDelay = UI_RETURN_DELAY;

      if (Mp3ReadRegister(SPI_BASS)!=newBassRegister){
	Mp3WriteRegister(SPI_BASS,newBassRegister>>8,newBassRegister&0xff);
      }

      //i = newBassRegister;
      //displayTitle[4] = lcd_hexchars[i>>12];
      //displayTitle[5] = lcd_hexchars[(i>>8)&0x0f];
      //displayTitle[6] = lcd_hexchars[(i>>4)&0x0f];
      //displayTitle[7] = lcd_hexchars[i&0x0f];
    

    }

    break; // UI_BASS and UI_TREBLE


  case UI_RECLEVEL:
    
    if (playingState==PS_RECORDING){
      if (KEY_FARRIGHT){
	uiReturnDelay = UI_RETURN_DELAY;
	if (reclevel++ == 100){
	  reclevel = 100;
	}
	Mp3WriteRegister(SPI_AICTRL1,
			 ((int)reclevel*32)>>8,((int)reclevel*32)&0xff);
	SPIPutChar(0);SPIWait();
      }
      
      if (KEY_FARLEFT){
	uiReturnDelay = UI_RETURN_DELAY;
	if (reclevel-- == 0){
	  reclevel = 0;
	}
	Mp3WriteRegister(SPI_AICTRL1,
			 ((int)reclevel*32)>>8,((int)reclevel*32)&0xff);
	SPIPutChar(0);SPIWait();
      }
    }else{
      //Not in recording
      if (KEY_FARRIGHT){
	//Enter recording mode
	playingState=PS_RECORDING; //request to record
      }
    }
    break; // UI_RECLEVEL

  case UI_STOP:
    
    if (KEY_FARRIGHT){
      playingState = PS_END_OF_SONG; /* Request to abort playing */
    }      
    break; // UI_STOP
    
  case UI_CUE:
    
    if (playingState == PS_NORMAL){ /* Only control when PS_NORMAL */
      if (KEY_FARRIGHT){
	uiReturnDelay = UI_RETURN_DELAY; /* Don't go back to title just yet */
	playingState = PS_CUE; /* Request */
      }      
      if (KEY_FARLEFT){
	uiReturnDelay = UI_RETURN_DELAY;
	playingState = PS_REWIND; /* Request */
      }      
    }

    break; // UI_SKIP

  case UI_END_OF_MODES:
    uiMode = UI_TITLE;
    break; // UI_END_OF_MODES
    

  } //End Switch


  // Draw screen
  UpdateDisplay();
}
  


/** Plays a disk file. Returns 1) if the file ends or 2) if the global
    variable playingState is not PS_NORMAL i.e. user has requested 
    stop or next or previous.*/
void PlayCurrentFile(){
  xdata char c, nFragments;

  playingState = PS_NORMAL; /* Request to play normally */
  //uiMode = UI_SPEC; /* User interface: show title SPECANA FOR VS1003*/

  LcdLocateHome();
  LcdPutConstantString("Opening ");

  ConsoleWrite ("Building file fragment table...");
  nFragments = BuildFragmentTable(); /* Too slow, rewrite! */
  ConsoleWrite("\rFragments: ");
  ConsolePutUInt(nFragments);

  LcdLocateHome();
  LcdPutConstantString("Playing ");

  for (c=0; c<nFragments; c++){
    sectorAddress.l = fragment[c].start;
    ConsoleWrite ("\rPlayer: Playing from sector ");
    ConsolePutUInt (sectorAddress.l);
    if (PlayDiskSectors(fragment[c].length)!=0){
      Mp3WriteRegister(SPI_MODE,0,SM_OUTOFWAV);
      SendZerosToVS10xx();
      return; //return without touching the value of playingState
    }
  }
  SendZerosToVS10xx();

  // After finishing normally default to requesting to play next song        
  playingState = PS_NEXT_SONG;

}



/** Program Entry Point.
 * In the final program, main() should be very small.
 * Currently it's the playground for developing the player
 * functionality. Most of it will be cleared away to new
 * functions soon. */
void main(){

  unsigned int currentFile;

  InitBoard();

  // Start "User Interface" timer
  ET0 = 1;
  EA = 1; 
  TR0 = 1;

  //LcdSplashScreen(); 
  InitDisplay (DS_STATIC,"        ","        ",0);
  
  ConsoleWrite ("\rVLSIPlayer\rStarting up.\r\r");
     
  LcdReset();
  LcdPutConstantString ("Filesys ");
  LcdLocateLine2();
  LcdPrintGenericResult (InitFileSystem());

  Mp3Reset();
  LoadPatch();
  StartPatch();

  // If left button is pressed during boot, enter recording.
  if (KEY_FARLEFT){
    while(KEY_LEFT)
      ;
    Record();
  }

  
#ifdef VS1003
  uiMode = UI_SPEC;   //For VS1003 default to SPECTRUM ANALYZER screen
#else
  uiMode = UI_TITLE;  //For others, default to TITLE screen
#endif
  
  playingState = PS_NEXT_SONG;
  currentFile = 1;

  while (1){
    ConsoleWrite("SPMax: ");ConsolePutHex8(SPMax);
    ConsoleWrite("PlayingState: ");ConsolePutHex8(playingState);

    // has someone requested to record?
    if (playingState == PS_RECORDING){
      uiMode = UI_TITLE;
      playingState = Record(); //record returns PS_NEXT_SONG or PS_RECORDING
      while (KEY_BUTTON)
	; //Wait until button is depressed
      ConsoleWrite("\rFinished recording.\r");      
      currentFile = 1;
      playingState == PS_NEXT_SONG;	
    }        

    if (OpenFile(currentFile)){
      currentFile = 1;
      if (OpenFile(currentFile)){
	LcdReset();
	LcdPutConstantString("No files.");
	while(1);
      }
    }
   
    LcdReset();
    LcdPutConstantString("File ");
    LcdPutUInt(currentFile);
    LcdLocateLine2();
    for (temp.c=0; temp.c<8; temp.c++){
      LcdPutChar(currentFileName[temp.c]);
    }
    
        
    while ((KEY_BUTTON)||(KEY_FARLEFT)||(KEY_FARRIGHT)||
	   (KEY_LEFT)||(KEY_RIGHT))
      ; /* Wait until no key is pressed */

    while (((!KEY_BUTTON)&&(!KEY_FARLEFT)&&(!KEY_FARRIGHT))
	   &&(playingState == PS_NORMAL))
      ; /* Wait untis some key is pressed or playing state not normal*/

    
    /* See if keystroke requests previous/next song */
    if (KEY_FARLEFT) currentFile--;
    if (KEY_FARRIGHT) currentFile++;
    if (currentFile==0) currentFile = 1;

    if (KEY_BUTTON || 
	(playingState==PS_NEXT_SONG) || 
	(playingState==PS_PREVIOUS_SONG)){
      while (KEY_BUTTON)
	;

      PlayCurrentFile();
      ConsoleWrite ("Playing state after playing is: ");
      ConsolePutUInt (playingState);
      
      if (playingState == PS_PREVIOUS_SONG) currentFile--;
      if (playingState == PS_NEXT_SONG) currentFile++;
      if (currentFile==0) currentFile = 1;
      if (playingState == PS_END_OF_SONG) playingState = PS_NORMAL;      
      
      
      Mp3SoftReset();
      LoadPatch();
      StartPatch();


      while ((KEY_BUTTON)||(KEY_FARLEFT)||(KEY_FARRIGHT)||
	     (KEY_LEFT)||(KEY_RIGHT))
	; /* Wait until no key is pressed */
    }

    
  }
   

}


